@@ -14,7 +14,7 @@ from hbshare.db.fund.product import ( get_authorized_product_list )
 
 from hbshare.db.simu.corp import (get_simu_corp_list_by_keyword, get_prod_list_by_corp_code)
 from hbshare.db.simu.nav import (get_simu_nav_by_code)
-from hbshare.db.simu.valuation import (get_prod_valuation_by_jjdm_gzrq)
+from hbshare.db.simu.valuation import (get_prod_valuation_by_jjdm_gzrq,get_valuation_holding_by_jjdm_gzrq)
 from hbshare.db.loader.data_query import (db_data_query)
 from hbshare.db.loader.data_save import (db_data_save)
 from hbshare.db.ApiUtils import commonQuery
