@@ -18,7 +18,7 @@ class DataApi:
     __token = ''
     __base_url = 'https://data.howbuy.com'
 
-    def __init__(self, token, timeout=10):
+    def __init__(self, token, timeout=300):
         """
         Parameters
         ----------
@@ -26,18 +26,21 @@ class DataApi:
             API接口TOKEN，用于用户认证
         """
         self.__token = token
+        if timeout and timeout > 600:
+            # 最大600
+            timeout = 600
         self.__timeout = timeout
 
-    def query(self, url, method='get', post_data={}):
+    def query(self, url, method='get', post_data={},params={}):
         headers = {
             "token": self.__token,
             "Referer": 'hbshare'
         }
 
         if method == 'get':
-            res = requests.get(url, headers=headers, timeout=self.__timeout)
+            res = requests.get(url, headers=headers, timeout=self.__timeout,params=params)
         else:
-            res = requests.post(url, json=post_data, headers=headers, timeout=self.__timeout)
+            res = requests.post(url, json=post_data, headers=headers, timeout=self.__timeout,params=params)
         if res.status_code != 200:
             res.raise_for_status()
         result = json.loads(res.text)
