@@ -69,9 +69,38 @@ class UrlEnum(Enum):
                                       'hb2n', 'hb3n', 'hb5n', 'hbdr', 'fl', 'pb', 'roe', 'gxl', 'pebfw', 'pbbfw', 'syl', 'nhsy1y', 'nhsy1n', 'nhsy2n',
                                       'nhsy3n', 'nhsy3y', 'nhsy6y', 'nhsy5n', 'gxlbfw', 'ltsz', 'zsz',)
                        )
-    ''' 查询指数行情 '''
+    ''' 查询指数行情收盘价格 '''
+    MARKET_SPJG = UrlCfg('http://%s/data/zs/spj', 'get', 'ams',
+                       lambda x: {},
+                        lambda y: {
+                           'dm': y['zqdm'],
+                           'startTime': y['startDate'],
+                           'endTime': y['endDate']
+                        },
+                       supportFields=('hb1n', 'hb1y', 'hb1z', 'hb2n', "hb3n","hb3y","hb5n",
+                                      "hb6y","hbcl","hbjn","jjsl","jyrq","scdm","spjg","tjrq","zqdm"
+                                      )
+                       )
+    ''' 查询指数行情收盘价格 '''
+    
 
 ################################ 公募 ################################
+    # 公募基金净值
+    FUND_JJJZ = UrlCfg('http://%s/data/gm/jz', 'get', 'ams',
+                       lambda x: {
+                           'dm': x['jjdm'],
+                            'startTime': x['startDate'],
+                            'endTime': x['endDate']
+                       },
+                       lambda y: {
+                           'dm': y['jjdm'],
+                            'startTime': y['startDate'],
+                            'endTime': y['endDate']
+                        },
+                       supportFields=('jzrq', 'jjdm', 'jjjz', 'ljjz', 'hbcl','hbdr','fqdwjz')
+                       )
+    '''公募基金净值'''
+
     # 公募基金回报
     FUND_JJHB = UrlCfg('http://%s/data/fund/brinson/jjhb', 'post', 'ams',
                        lambda x: {
@@ -227,6 +256,21 @@ class UrlEnum(Enum):
                                      'hbcl', 'hbfh', 'fqdwjz',)
                       )
     ''' 私募基金回报数据 '''
+    # 私募基金净值
+    SIMU_JJJZ = UrlCfg('http://%s/data/sm/jz', 'get', 'ams',
+                       lambda x: {
+                           'dm': x['jjdm'],
+                            'startTime': x['startDate'],
+                            'endTime': x['endDate']
+                       },
+                       lambda y: {
+                           'dm': y['jjdm'],
+                            'startTime': y['startDate'],
+                            'endTime': y['endDate']
+                        },
+                       supportFields=('jzrq', 'jjdm', 'jjjz', 'ljjz', 'hbcl','hbdr','fqdwjz')
+                       )
+    '''私募基金净值'''
 
 ################################ A股市场/科创板 ################################
     # A股市场估值数据
@@ -320,7 +364,7 @@ class UrlEnum(Enum):
                       )
     '''科创板复权行情数据'''
     # 非标数据日净值 - 日净值分页查询
-    FB_RJZ = UrlCfg('http://%s/admin/dailyNetValue/queryList', 'post', 'ams-admin',
+    FB_RJZ = UrlCfg('https://%s/admin/dailyNetValue/queryList', 'post', 'ams-admin',
                     lambda x: {
                         'belongDate': {
                             'startDate': x.get('startDate'),
@@ -340,7 +384,7 @@ class UrlEnum(Enum):
                     )
     '''非标数据日净值 - 日净值分页查询'''
     # fof估值子基金 - 查询子基金列表
-    FOF_ZJJLIST = UrlCfg('http://%s/admin/fof/gz/sf/list', 'post', 'ams-admin',
+    FOF_ZJJLIST = UrlCfg('https://%s/admin/fof/gz/sf/list', 'post', 'ams-admin',
                          lambda x: {
                              'jzrq': {
                                  'startDate': x.get('startDate'),
@@ -363,7 +407,7 @@ class UrlEnum(Enum):
                          )
     '''fof估值子基金 - 查询子基金列表'''
         # fof历史净值 - 查询历史净值
-    FOF_LSJZ = UrlCfg('http://%s/admin/sm/fof/sy/lsjz', 'get', 'ams-admin',
+    FOF_LSJZ = UrlCfg('https://%s/admin/sm/fof/sy/lsjz', 'get', 'ams-admin',
                          lambda x: {},
                          lambda y: {'access_token': y['access_token'],'dm':y['jjdm']},
                          supportFields=('jzrq', 'ljjz', 'jjjz')
