@@ -81,7 +81,7 @@ def commonQuery(urlEnum, pause=0.01, timeout=10, **kwargs):
 
 if __name__ == '__main__':
 
-    data = commonQuery('FOF_LSJZ',
-                       jjdm='SER337', access_token='1f84d0cb895e4a458854ab64e16016f1',
-                       fields=['jzrq','ljjz','jjjz'])
+    data = commonQuery('SIMU_JJJZ',
+                       jjdm='P10975',
+                       startDate= '19900104',endDate='20221104')
     print(data)
