@@ -34,26 +34,23 @@ class Loader:
 
     def read_private_index_daily_k_given_indexs(self, indexs, start, end):
         indexs = "'" + "','".join(indexs) + "'"
-        sql = "SELECT zsdm AS INDEX_CODE, tjyf AS TRADE_MONTH, spjg AS CLOSE_INDEX FROM st_hedge.t_st_sm_hmzs WHERE zsdm in ({0}) AND tjyf >= {1} AND tjyf <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
+        sql = "SELECT zsdm AS INDEX_CODE, jyrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_hedge.t_st_sm_zhmzs WHERE zsdm in ({0}) AND jyrq >= {1} AND jyrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
         df = self.get_df(sql, db='highuser', page_size=200000)
         return df
 
-    def read_market_index_daily_k_given_indexs(self, indexs, start, end):
-        indexs = "'" + "','".join(indexs) + "'"
-        sql = "SELECT zqdm AS INDEX_CODE, jyrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_market.t_st_zs_hqql WHERE zqdm in ({0}) AND jyrq >= {1} AND jyrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
-        df = self.get_df(sql, db='alluser', page_size=200000)
+    def read_market_index_daily_k_given_index(self, index, start, end):
+        df = hbs.commonQuery('MARKET_SPJG', zqdm=index, startDate=start, endDate=end, fields=['zqdm', 'jyrq', 'spjg'])
+        df = df.rename(columns={'zqdm': 'INDEX_CODE', 'jyrq': 'TRADE_DATE', 'spjg': 'CLOSE_INDEX'})
         return df
 
-    def read_mutual_fund_cumret_given_codes(self, codes, start, end):
-        codes = "'" + "','".join(codes) + "'"
-        sql = "SELECT jjdm AS FUND_CODE, jzrq AS TRADE_DATE, fqdwjz AS ADJ_NAV FROM st_fund.t_st_gm_rhb WHERE jjdm in ({0}) AND jzrq >= {1} AND jzrq <= {2} AND m_opt_type <> '03'".format(codes, start, end)
-        df = self.get_df(sql, db='funduser', page_size=200000)
+    def read_mutual_fund_cumret_given_code(self, code, start, end):
+        df = hbs.commonQuery('FUND_JJJZ', jjdm=code, startDate=start, endDate=end, fields=['jjdm', 'jzrq', 'fqdwjz'])
+        df = df.rename(columns={'jjdm': 'FUND_CODE', 'jzrq': 'TRADE_DATE', 'fqdwjz': 'ADJ_NAV'})
         return df
 
-    def read_private_fund_adj_nav_given_codes(self, codes, start, end):
-        codes = "'" + "','".join(codes) + "'"
-        sql = "SELECT jjdm AS FUND_CODE, jzrq AS TRADE_DATE, fqdwjz AS ADJ_NAV FROM st_hedge.t_st_rhb WHERE jjdm in ({0}) AND jzrq >= {1} AND jzrq <= {2} AND m_opt_type <> '03'".format(codes, start, end)
-        df = self.get_df(sql, db='highuser', page_size=200000)
+    def read_private_fund_adj_nav_given_code(self, code, start, end):
+        df = hbs.commonQuery('SIMU_JJJZ', jjdm=code, startDate=start, endDate=end, fields=['jjdm', 'jzrq', 'fqdwjz'])
+        df = df.rename(columns={'jjdm': 'FUND_CODE', 'jzrq': 'TRADE_DATE', 'fqdwjz': 'ADJ_NAV'})
         return df
 
     def read_market_index_market_value_given_indexs(self, indexs, start, end):
@@ -70,25 +67,18 @@ class Loader:
 
 
 if __name__ == '__main__':
-    import time
-    import pandas as pd
+    import hbshare as hbs
 
-    df = pd.read_csv('D:/Git/hbshare/hbshare/fe/xwq/data/fund.csv')
-    df['FUND_CODE'] = df['FUND_CODE'].apply(lambda x: str(x).zfill(6))
-    codes = df['FUND_CODE'].unique().tolist()[:40]
-    t1 = time.time()
-    data = Loader().read_mutual_fund_cumret_given_codes(codes, '20121027', '20221027')
-    t2 = time.time()
-    print(t2 - t1)
-    data = data.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
-    pass
+    data = hbs.commonQuery('FUND_JJJZ',
+                           jjdm='688888',
+                           startDate='20230101', endDate='20230228')
 
-    df = pd.read_csv('D:/Git/hbshare/hbshare/fe/xwq/data/index_aam.csv')
-    df['INDUSTRY_ID'] = df['INDUSTRY_ID'].apply(lambda x: str(x).zfill(6))
-    indexs = df['INDUSTRY_ID'].unique().tolist() + ['000300', '000905', '000906', 'CBA00301', '000852', '399001', '885001', '881001', '399006']
-    t3 = time.time()
-    data = Loader().read_market_index_daily_k_given_indexs(indexs, '20171027', '20221027')
-    t4 = time.time()
-    print(t4 - t3)
-    data = data.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
-    pass
\ No newline at end of file
+    data = hbs.commonQuery('SIMU_JJJZ',
+                       jjdm='SX8958',
+                       startDate='20230101', endDate='20230228')
+
+    data = hbs.commonQuery('MARKET_SPJG',
+                       zqdm='000300',
+                       startDate='20230101', endDate='20230228')
+
+    print(data)
\ No newline at end of file
