@@ -5,6 +5,7 @@ from hbshare.fe.asset_allocation.data_loader import Loader
 from hbshare.fe.common.util.exception import InputParameterError
 from hbshare.fe.common.util.logger import logger
 from hbshare.fe.common.util.verifier import verify_type
+from multiprocessing import Pool
 from numpy import linalg as la
 import numpy as np
 import pandas as pd
@@ -44,7 +45,7 @@ def correct_cov(cov):
 
 class RiskParityBudget:
     def __init__(self, asset_type, asset_list, method, start_date, end_date, is_reblance, reblance_type, n, frequency, compute_risk_days,
-                 lb_list=None, ub_list=None, total_weight=None, risk_budget_list=None):
+                 lb_list=None, ub_list=None, total_weight=None, risk_budget_list=None, processes=16):
         self.asset_type = asset_type
         self.asset_list = asset_list
         self.method = method
@@ -60,6 +61,7 @@ class RiskParityBudget:
         self.total_weight = 1.0 if total_weight is None else total_weight
         self.risk_budget_list = [1.0 / len(self.asset_list)] * len(self.asset_list) if risk_budget_list is None or risk_budget_list == [] else risk_budget_list
         self.risk_budget_list = [budget / sum(self.risk_budget_list) for budget in self.risk_budget_list]
+        self.processes = processes
         self._verify_input_param()
         self._load()
         self._init_data()
@@ -79,6 +81,7 @@ class RiskParityBudget:
         verify_type(self.ub_list, 'ub_list', list)
         verify_type(self.total_weight, 'total_weight', float)
         verify_type(self.risk_budget_list, 'risk_budget_list', list)
+        verify_type(self.processes, 'processes', int)
         if self.asset_type not in ['index', 'fund']:
             msg = "asset_type not supported, check your input"
             logger.error(msg)
@@ -131,6 +134,10 @@ class RiskParityBudget:
             msg = "risk_budget must be between 0 and 1, check your input"
             logger.error(msg)
             raise InputParameterError(message=msg)
+        if self.processes <= 0:
+            msg = "processes must be larger than 0, check your input"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
 
     def _load(self):
         self.calendar_df = Loader().read_cal('19900101', self.end_date)
@@ -164,16 +171,19 @@ class RiskParityBudget:
             self.mutual_index_nav_df = self.mutual_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
             self.mutual_index_nav_df = self.mutual_index_nav_df.sort_index()
 
-            self.private_index_nav_df = Loader().read_private_index_daily_k_given_indexs(self.asset_list, self.start_date_backup[:6], self.end_date[:6])
-            self.private_index_nav_df = self.private_index_nav_df[['INDEX_CODE', 'TRADE_MONTH', 'CLOSE_INDEX']] if len(self.private_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_MONTH', 'CLOSE_INDEX'])
+            self.private_index_nav_df = Loader().read_private_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
+            self.private_index_nav_df = self.private_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.private_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
             self.private_index_nav_df = self.private_index_nav_df.drop_duplicates()
-            self.private_index_nav_df['TRADE_MONTH'] = self.private_index_nav_df['TRADE_MONTH'].astype(str)
-            self.private_index_nav_df = self.private_index_nav_df.merge(self.calendar_df[self.calendar_df['IS_MONTH_END'] == 1][['YEAR_MONTH', 'TRADE_DATE']].rename(columns={'YEAR_MONTH': 'TRADE_MONTH'}), on=['TRADE_MONTH'], how='left')
+            self.private_index_nav_df['TRADE_DATE'] = self.private_index_nav_df['TRADE_DATE'].astype(str)
             self.private_index_nav_df = self.private_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
             self.private_index_nav_df = self.private_index_nav_df.sort_index()
 
-            self.market_index_nav_df = Loader().read_market_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
-            self.market_index_nav_df = self.market_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.market_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
+            market_index_nav_df_list = []
+            for index in self.asset_list:
+                market_index_nav_df = Loader().read_market_index_daily_k_given_index(index, self.start_date_backup, self.end_date)
+                market_index_nav_df = market_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(market_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
+                market_index_nav_df_list.append(market_index_nav_df)
+            self.market_index_nav_df = pd.concat(market_index_nav_df_list)
             self.market_index_nav_df = self.market_index_nav_df.drop_duplicates()
             self.market_index_nav_df['TRADE_DATE'] = self.market_index_nav_df['TRADE_DATE'].astype(str)
             self.market_index_nav_df = self.market_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
@@ -182,15 +192,23 @@ class RiskParityBudget:
             self.nav_df = pd.concat([self.mutual_index_nav_df, self.private_index_nav_df, self.market_index_nav_df], axis=1)
 
         if self.asset_type == 'fund':
-            self.mutual_fund_nav_df = Loader().read_mutual_fund_cumret_given_codes(self.asset_list, self.start_date_backup, self.end_date)
-            self.mutual_fund_nav_df = self.mutual_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(self.mutual_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
+            mutual_fund_nav_df_list = []
+            for code in self.asset_list:
+                mutual_fund_nav_df = Loader().read_mutual_fund_cumret_given_code(code, self.start_date_backup, self.end_date)
+                mutual_fund_nav_df = mutual_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(mutual_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
+                mutual_fund_nav_df_list.append(mutual_fund_nav_df)
+            self.mutual_fund_nav_df = pd.concat(mutual_fund_nav_df_list)
             self.mutual_fund_nav_df = self.mutual_fund_nav_df.drop_duplicates()
             self.mutual_fund_nav_df['TRADE_DATE'] = self.mutual_fund_nav_df['TRADE_DATE'].astype(str)
             self.mutual_fund_nav_df = self.mutual_fund_nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
             self.mutual_fund_nav_df = self.mutual_fund_nav_df.sort_index()
 
-            self.private_fund_nav_df = Loader().read_private_fund_adj_nav_given_codes(self.asset_list, self.start_date_backup, self.end_date)
-            self.private_fund_nav_df = self.private_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(self.private_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
+            private_fund_nav_df_list = []
+            for code in self.asset_list:
+                private_fund_nav_df = Loader().read_private_fund_adj_nav_given_code(code, self.start_date_backup, self.end_date)
+                private_fund_nav_df = private_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(private_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
+                private_fund_nav_df_list.append(private_fund_nav_df)
+            self.private_fund_nav_df = pd.concat(private_fund_nav_df_list)
             self.private_fund_nav_df = self.private_fund_nav_df.drop_duplicates()
             self.private_fund_nav_df['TRADE_DATE'] = self.private_fund_nav_df['TRADE_DATE'].astype(str)
             self.private_fund_nav_df = self.private_fund_nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
@@ -253,8 +271,8 @@ class RiskParityBudget:
         # 确定每个再平衡时点所需数据
         trading_day_list = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['TRADE_DATE'].unique().tolist()
         if self.asset_type == 'index' and len(self.private_index_nav_df) > 0:
-            date_list = self.calendar_df[self.calendar_df['IS_MONTH_END'] == 1]['TRADE_DATE'].unique().tolist()
-            self.q = 12
+            date_list = self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['TRADE_DATE'].unique().tolist()
+            self.q = 52
         elif self.asset_type == 'fund' and len(self.private_fund_nav_df) > 0:
             date_list = self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['TRADE_DATE'].unique().tolist()
             self.q = 52
@@ -297,42 +315,53 @@ class RiskParityBudget:
         #     print('no enough nav data for computing risk model!')
         return
 
-    def get_all(self):
-        weight_list, status_list = [], []
-        for date, period_data in self.preprocessed_data.items():
-            if period_data['cov_df'].empty:
+    def get_optimal_one_process(self, date, period_data):
+        if period_data['cov_df'].empty:
+            weight = pd.DataFrame(index=[date], columns=self.asset_list + ['cash'])
+            status = pd.DataFrame(index=[date], columns=['优化状态'], data=['infeasible'])
+            status = status.merge(self.reason_df, left_index=True, right_index=True, how='left')
+        else:
+            try:
+                if self.method == 'risk_parity':
+                    self.cov = period_data['cov_df']
+                    weight, status = RiskParity(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight).solve()
+                    weight = pd.DataFrame(weight, columns=[date]).T
+                    reason = '{0}：模型当期无优化解！'.format(date) if status == 'infeasible' else np.nan
+                    status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
+                else:
+                    self.cov = period_data['cov_df']
+                    weight, status = RiskBudget(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight, self.risk_budget_list).solve()
+                    weight = pd.DataFrame(weight, columns=[date]).T
+                    reason = '{0}：模型当期无优化解！'.format(date) if status == 'infeasible' else np.nan
+                    status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
+            except:
                 weight = pd.DataFrame(index=[date], columns=self.asset_list + ['cash'])
-                status = pd.DataFrame(index=[date], columns=['优化状态'], data=['infeasible'])
-                status = status.merge(self.reason_df, left_index=True, right_index=True, how='left')
-            else:
-                try:
-                    if self.method == 'risk_parity':
-                        self.cov = period_data['cov_df']
-                        weight, status = RiskParity(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight).solve()
-                        weight = pd.DataFrame(weight, columns=[date]).T
-                        reason = '{0}：模型当期无优化解！' if status == 'infeasible' else np.nan
-                        status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
-                    else:
-                        self.cov = period_data['cov_df']
-                        weight, status = RiskBudget(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight, self.risk_budget_list).solve()
-                        weight = pd.DataFrame(weight, columns=[date]).T
-                        reason = '{0}：模型当期无优化解！' if status == 'infeasible' else np.nan
-                        status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
-                except:
-                    weight = pd.DataFrame(index=[date], columns=self.asset_list + ['cash'])
-                    status = pd.DataFrame(index=[date], columns=['优化状态'], data=['infeasible'])
-                    status = status.merge(self.reason_df, left_index=True, right_index=True, how='left')
+                status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[['infeasible', '{0}：模型当期无优化解！'.format(date)]])
+        print('[{0}]/[{1}] done'.format(date, self.method))
+        return weight, status
+
+    def get_all(self):
+        result_list = []
+        p = Pool(processes=self.processes)
+        for date, period_data in self.preprocessed_data.items():
+            result = p.apply_async(self.get_optimal_one_process, args=(date, period_data))
+            result_list.append(result)
+        p.close()
+        p.join()
+        print('processes done.')
+        weight_list, status_list = [], []
+        for result in result_list:
+            weight, status = result.get()
             weight_list.append(weight)
             status_list.append(status)
-            print('[{0}]/[{1}] done'.format(date, self.method))
-        weight_df = pd.concat(weight_list)
-        status_df = pd.concat(status_list)
+        weight_df = pd.concat(weight_list).sort_index()
+        status_df = pd.concat(status_list).sort_index()
         # 权重归一化（本版本不支持有现金配置）
         weight_df = weight_df[self.asset_list]
         weight_df['TOTAL'] = weight_df.sum(axis=1)
         weight_df['TOTAL'] = weight_df['TOTAL'].replace(0.0, np.nan)
         weight_df = weight_df.apply(lambda x: x.iloc[:len(self.asset_list)] / x[-1], axis=1)
-        if self.reblance_type == 'init_weight':
+        if self.reblance_type == 'init_weight' and len(weight_df.dropna()) != 0:
             start_index = weight_df.dropna().index[0]
             weight_df.loc[weight_df.index > start_index, :] = np.nan
             weight_df = weight_df.fillna(method='ffill')
@@ -345,7 +374,7 @@ class RiskParityBudget:
 
 if __name__ == '__main__':
     # mutual_index: ['HM0001', 'HM0024', 'HM0095']
-    # private_index: ['HB0000', 'HB0016', 'HB1001']
+    # private_index: ['HB0011', 'HB0016', 'HB1001']
     # market_index: ['000300', '000906', 'CBA00301']
     # mutual_fund: ['002943', '688888', '000729']
     # private_fund: ['SGK768', 'SX8958', 'SR4480']
@@ -359,6 +388,7 @@ if __name__ == '__main__':
                                             reblance_type='init_target',  # init_weight, init_target
                                             n=3,
                                             frequency='month',  # day, week, month, quarter, year
-                                            compute_risk_days=120  # 交易日
+                                            compute_risk_days=120,  # 交易日
+                                            processes=4
                                             ).get_all()
     print(weight_df, status_df)
\ No newline at end of file
