@@ -231,11 +231,12 @@ class NavAttributionLoader:
 
 
 class CtaNavAttributionLoader:
-    def __init__(self, fund_id, start_date, end_date, style_factors):
+    def __init__(self, fund_id, start_date, end_date, factor_dict, table_name):
         self.fund_id = fund_id
         self.start_date = start_date
         self.end_date = end_date
-        self.style_factors = style_factors
+        self.factor_dict = factor_dict
+        self.table_name = table_name
         self._pre_date_process()
 
     def _pre_date_process(self):
@@ -285,14 +286,14 @@ class CtaNavAttributionLoader:
         return data.set_index('tradeDate')['adj_nav']
 
     def _load_factor_data(self):
-        sql_script = "SELECT * FROM cta_index where TDATE >= {} and TDATE <= {}".format(
-            self.pre_date, self.end_date)
+        sql_script = "SELECT * FROM {} where TDATE >= {} and TDATE <= {}".format(
+            self.table_name, self.pre_date, self.end_date)
         engine = create_engine(engine_params)
         factor_data = pd.read_sql(sql_script, engine).rename(columns={"TDATE": "TRADEDATE"})
         factor_data['TRADEDATE'] = \
             factor_data['TRADEDATE'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
         factor_data = pd.pivot_table(
-            factor_data, index='TRADEDATE', columns='FACTOR', values='CLOSE').sort_index()[self.style_factors]
+            factor_data, index='TRADEDATE', columns='FACTOR', values='CLOSE').sort_index()[list(self.factor_dict.keys())]
 
         return factor_data
 
