@@ -4,11 +4,13 @@
 import numpy as np
 import pandas as pd
 import statsmodels.api as sm
+from datetime import datetime
 from hbshare.fe.common.util.verifier import verify_type
 from hbshare.fe.common.util.exception import InputParameterError, PreprocessError
 from hbshare.fe.common.util.logger import logger
 from hbshare.fe.common.util.data_loader import NavAttributionLoader
 from hbshare.fe.common.util.regressions import Regression
+from hbshare.fe.common.util.helper_func import get_last_day_of_the_quarter
 
 
 class StyleAttributionOutputs:
@@ -293,6 +295,15 @@ class StyleAttribution:
 
         return attr_summary, r_square_series, exposure_series, return_attr_series, total_attr_df, total_return_df
 
+    @staticmethod
+    def day_adjust(df):
+        cols = df.columns
+        df['dt'] = pd.to_datetime(df['date'])
+        df['dt_adjust'] = df['dt'].map(get_last_day_of_the_quarter)
+        df['date'] = df['dt_adjust'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
+
+        return df[cols]
+
     def get_all(self, processed=True):
         output = StyleAttributionOutputs()
 
@@ -330,6 +341,10 @@ class StyleAttribution:
             total_return_df = total_return_df[
                 ['fund_id', 'tradeDate', 'attr_type', 'portfolio', 'benchmark', 'dynamic_style_bm']]
 
+            # Day Adjustment
+            attr_df = self.day_adjust(attr_df)
+            total_attr_df = self.day_adjust(total_attr_df)
+
             output.attribution_summary = attr_summary
             output.attr_df = attr_df
             output.style_allo_bar = total_attr_df
@@ -342,17 +357,13 @@ class StyleAttribution:
             output.style_allo_bar = total_attr_df
             output.style_allo_line = total_return_df
 
-        # import pickle
-        # with open('D:\\kevin\\nav_attribution_sample_data.pickle', 'wb') as f:
-        #     pickle.dump(output.to_dict(), f)
-
         return output.to_dict()
 
 
 if __name__ == '__main__':
     # 公募样例：110011
     # res = StyleAttribution(fund_id='002943', fund_type='mutual', start_date='20161220', end_date='20220105',
-    #                        factor_type='style_allo', benchmark_id='000300', nav_frequency='day').get_all(processed=False)
+    #                        factor_type='style_allo', benchmark_id='000300', nav_frequency='day').get_all(processed=True)
     # 私募样例
     res = StyleAttribution(fund_id='6E0108', fund_type='private', start_date='20181211', end_date='20220915',
                            factor_type='style_allo', benchmark_id='000985', nav_frequency='week').get_all()
