@@ -288,19 +288,35 @@ class FEDB:
         return df
 
     def read_hbs_size_property_given_code(self, code):
-        size_list = ['大盘', '中盘', '小盘']
+        size_list = ['大盘', '中小盘']
         size = ','.join(size_list)
-        size_rank_list = ['大盘_rank', '中盘_rank', '小盘_rank']
+        size_rank_list = ['大盘_rank', '中小盘_rank']
         size_rank = ','.join(size_rank_list)
         sql = "select jjdm, asofdate, shift_lv, cen_lv, {0}, shift_lv_rank, cen_lv_rank, {1} from fe_temp_data.hbs_size_property where jjdm='{2}' and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_size_property)".format(size, size_rank, code)
         df = self.get_df(sql)
         return df
 
+    def read_hbs_size_property_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        size_list = ['大盘', '中小盘']
+        size = ','.join(size_list)
+        size_rank_list = ['大盘_rank', '中小盘_rank']
+        size_rank = ','.join(size_rank_list)
+        sql = "select jjdm, asofdate, shift_lv, cen_lv, {0}, shift_lv_rank, cen_lv_rank, {1} from fe_temp_data.hbs_size_property where jjdm in ({2}) and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_size_property)".format(size, size_rank, codes)
+        df = self.get_df(sql)
+        return df
+
     def read_size_exposure_given_code(self, code):
         sql = "select jjdm, jsrq, size_type, zjbl from fe_temp_data.hbs_size_exp where jjdm='{0}'".format(code)
         df = self.get_df(sql)
         return df
 
+    def read_size_exposure_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jsrq, size_type, zjbl from fe_temp_data.hbs_size_exp where jjdm in ({0})".format(codes)
+        df = self.get_df(sql)
+        return df
+
     def read_hbs_style_property_given_code(self, code):
         style_list = ['成长', '价值']
         style = ','.join(style_list)
@@ -310,11 +326,27 @@ class FEDB:
         df = self.get_df(sql)
         return df
 
+    def read_hbs_style_property_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        style_list = ['成长', '价值']
+        style = ','.join(style_list)
+        style_rank_list = ['成长_rank', '价值_rank']
+        style_rank = ','.join(style_rank_list)
+        sql = "select jjdm, asofdate, shift_lv, cen_lv, {0}, shift_lv_rank, cen_lv_rank, {1} from fe_temp_data.hbs_style_property where jjdm in ({2}) and asofdate=(SELECT max(asofdate) FROM fe_temp_data.hbs_style_property)".format(style, style_rank, codes)
+        df = self.get_df(sql)
+        return df
+
     def read_style_exposure_given_code(self, code):
         sql = "select jjdm, jsrq, style_type, zjbl from fe_temp_data.hbs_style_exp where jjdm='{0}'".format(code)
         df = self.get_df(sql)
         return df
 
+    def read_style_exposure_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jsrq, style_type, zjbl from fe_temp_data.hbs_style_exp where jjdm in ({0})".format(codes)
+        df = self.get_df(sql)
+        return df
+
     def read_hbs_size_style_property_history_given_code(self, code):
         sql = "select jjdm, jsrq, shift_ratio_size, c_level_size, shift_ratio_style, c_level_style from fe_temp_data.hbs_cen_shift_ratio_his_style where jjdm='{0}'".format(code)
         df = self.get_df(sql)
@@ -374,6 +406,12 @@ class FEDB:
         df = self.get_df(sql)
         return df
 
+    def read_hbs_rank_history_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jsrq, shift_ratio_size, c_level_size, shift_ratio_style, c_level_style, shift_ratio_theme, c_level_theme, shift_ratio_ind, c_level_ind, c_level_stock from fe_temp_data.hbs_rank_his where jjdm in ({0})".format(codes)
+        df = self.get_df(sql)
+        return df
+
     def read_hbs_his_data(self, table_name):
         sql = "select * from fe_temp_data.{0}".format(table_name)
         df = self.get_df(sql)
@@ -393,4 +431,20 @@ class FEDB:
         cols = ','.join(cols)
         sql = "select {0} from fe_temp_data.{1} where INDUSTRY_TYPE={2}".format(cols, table_name, sw_type)
         df = self.get_df(sql)
+        return df
+
+    def read_timing_data(self, cols, table_name, start, end):
+        cols = ','.join(cols)
+        sql = "select {0} from fe_temp_data.{1} where TRADE_DATE >= '{2}' AND TRADE_DATE <= '{3}'".format(cols, table_name, start, end)
+        df = self.get_df(sql)
+        return df
+
+    def read_size_index(self):
+        sql = "SELECT * from size_index_history"
+        df = self.get_df(sql)
+        return df
+
+    def read_style_index(self):
+        sql = "SELECT * from style_index_history"
+        df = self.get_df(sql)
         return df
\ No newline at end of file
