@@ -37,7 +37,7 @@ class HBDB:
         return df
 
     def read_stock_fund_info(self):
-        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, ejfl, kffb from st_fund.t_st_gm_jjxx where jjzt='0' and cpfl='2' and ejfl in ('13', '35', '37')"
+        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, ejfl, kffb from st_fund.t_st_gm_jjxx where jjzt='0' and cpfl='2' and ejfl in ('13', '35', '37') and m_opt_type<>'03'"
         df = self.get_df(sql, db='funduser')
         return df
 
@@ -97,6 +97,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_holder_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jsrq, ggrq, grcyfe, grcybl, jgcyfe, jgcybl, ygcyfe, ygcybl from st_fund.t_st_gm_fecyrbd where jjdm in ({0}) and m_opt_type<>'03'".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_fund_manager_product_given_code(self, manager_code):
         sql = "select rydm, jsrq, zgcpsl, zgcpgm, dbjj from st_fund.t_st_gm_jlzgcptj where rydm='{0}'".format(manager_code)
         df = self.get_df(sql, db='funduser')
@@ -112,6 +118,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_position_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jsrq, jjzzc, jjjzc, gptzsz, gptzzjb, zqzsz, zqzszzjb, jjtzszhj, jjtzhjzjb, hbzjsz, hbzjzjb from st_fund.t_st_gm_zcpz where jjdm in ({0}) and m_opt_type<>'03'".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_fund_position_given_date(self, date):
         sql = "select jjdm, jsrq, jjzzc, gptzsz, zqzsz, jjtzszhj, hbzjsz from st_fund.t_st_gm_zcpz where jsrq='{0}'".format(date)
         df = self.get_df(sql, db='funduser')
@@ -133,6 +145,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_turnover_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jsrq, tjqj, hsl from st_fund.t_st_gm_jjhsl where jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_fund_nav_given_code(self, code):
         sql = "select jjdm, jzrq, jjjz, ljjz from st_fund.t_st_gm_jjjz where jjdm='{0}'".format(code)
         df = self.get_df(sql, db='funduser')
@@ -163,6 +181,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_return_period_given_codes_and_date(self, codes, date):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jzrq, zblb, rqnp, zbnp, zbnhnp from st_fund.t_st_gm_rqjhb where m_opt_type <> '03' and jjdm in ({0}) and jzrq = '{1}'".format(codes, date)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_private_fund_return_period_given_code_and_date(self, code, date):
         sql = "select jjdm, jzrq, zblb, rqnp, zbnp, nhzbnp from st_hedge.t_st_sm_qjhb_zx where m_opt_type <> '03' and jjdm='{0}' and jzrq >= '{1}'".format(code, date)
         df = self.get_df(sql, db='highuser')
@@ -183,6 +207,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_maxdrawdown_period_given_codes_and_date(self, codes, date):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, jzrq, zblb, zbnp, zdhchfts, zdhccxts from st_fund.t_st_gm_rqjzdhc where m_opt_type <> '03' and jjdm in ({0}) and jzrq = '{1}'".format(codes, date)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_private_fund_maxdrawdown_period_given_code_and_date(self, code, date):
         sql = "select jjdm, jzrq, zblb, zbnp, hfts from st_hedge.t_st_sm_qjzdhc_zx where m_opt_type <> '03' and jjdm='{0}' and jzrq >= '{1}'".format(code, date)
         df = self.get_df(sql, db='highuser')
@@ -223,6 +253,48 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_return_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, rqzh, rq1n, hb1n from st_fund.t_st_gm_nhb where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
+    def read_fund_maxdrawdown_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, zbnp1n, zdhchfts1n, zdhccxts1n from st_fund.t_st_gm_nzdhc_ls where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
+    def read_fund_volatility_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, zzbnp1n, znhzbnp1n from st_fund.t_st_gm_nbdl_ls where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
+    def read_fund_sharpe_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, zzbnp1n, znhzbnp1n from st_fund.t_st_gm_nxpbl_ls where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
+    def read_fund_calmer_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, zbnp1n from st_fund.t_st_gm_nkmbl_ls where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
+    def read_fund_winratio_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, zbnp1n from st_fund.t_st_gm_ntzsl_ls where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
+    def read_fund_pl_year_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjnf, zsypj1n, fsypj1n, pjsyb1n from st_fund.t_st_gm_npjsyb_ls where m_opt_type <> '03' and jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_private_fund_return_year_given_code(self, code):
         sql = "select jjdm, tjnf, hblb, rqzh, rq1n, hb1n, nhhb1n from st_hedge.t_st_sm_nhb_ls where m_opt_type <> '03' and jjdm='{0}'".format(code)
         df = self.get_df(sql, db='highuser')
@@ -268,6 +340,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser')
         return df
 
+    def read_fund_brinson_attribution_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select jjdm, tjrq, portfolio_return, benchmark_return, asset_allo, sector_allo, equity_selection, trading, portfolio_weight, benchmark_weight from st_fund.r_st_hold_excess_attr_df where jjdm in ({0})".format(codes)
+        df = self.get_df(sql, db='funduser')
+        return df
+
     def read_fund_barra_attribution_given_code(self, code):
         sql = "select jjdm, tjrq, attr_type, style_factor, data_type, data_value from st_fund.r_st_nav_attr_df where jjdm='{0}'".format(code)
         df = self.get_df(sql, db='funduser')
@@ -422,6 +500,12 @@ class HBDB:
         df = self.get_df(sql, db='alluser', page_size=200000)
         return df
 
+    def read_index_daily_k_given_date(self, date, indexs):
+        indexs = '"' + '","'.join(indexs) + '"'
+        sql = "select scdm, zqdm, zqmc, jyrq, qspj, kpjg, spjg, zgjg, zdjg, zdsl, zdfd, bdfd, cjsl, cjjs, cjbs, ltsz, zsz, pe, pb, roe, gxl from st_market.t_st_zs_hqql where jyrq='{0}' and zqdm in ({1}) ".format(date, indexs)
+        df = self.get_df(sql, db='alluser', page_size=200000)
+        return df
+
     def read_industry_index_given_date_and_names(self, date, names):
         names = '"' + '","'.join(names) + '"'
         sql = "select zqmc, zqdm from st_market.t_st_zs_hqql where jyrq='{0}' and zqmc in ({1}) and left(zqdm, 1)='8' and length(zqdm)=6".format(date, names)
@@ -528,6 +612,54 @@ class HBDB:
         df = self.get_df(sql, db='alluser', page_size=200000)
         return df
 
+    def read_stock_daily_k_ch(self, date):
+        sql = "SELECT TDATE, SYMBOL, SNAME, TCLOSE, PCHG, VATURNOVER, TURNOVER, MCAP, TCAP FROM finchina.CHDQUOTE WHERE TDATE = {0}".format(date)
+        df = self.get_df(sql, db='readonly', page_size=200000)
+        return df
+
+    def read_index_daily_k_ch(self, date):
+        sql = "SELECT TDATE, SYMBOL, SNAME, TCLOSE, PCHG, VATURNOVER FROM finchina.CIHDQUOTE WHERE TDATE = {0}".format(date)
+        df = self.get_df(sql, db='readonly', page_size=200000)
+        return df
+
+    def read_stock_valuation_jy(self, date):
+        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE, a.PE AS PE_TTM, a.PB AS PB_LF, a.PCFTTM AS PCF_TTM, a.DividendRatio AS DIVIDEND_RATIO_TTM from hsjy_gg.LC_DIndicesForValuation a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
+        df = self.get_df(sql, db='readonly', page_size=200000)
+        return df
+
+    def read_star_stock_valuation_jy(self, date):
+        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE, a.PETTM AS PE_TTM, a.PB AS PB_LF, a.PCFTTM AS PCF_TTM, a.DividendRatioTTM AS DIVIDEND_RATIO_TTM from hsjy_gg.LC_STIBDIndiForValue a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
+        df = self.get_df(sql, db='readonly', page_size=200000)
+        return df
+
+    def read_stock_finance_jy(self, date):
+        sql = "SELECT b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.InfoPublDate, 'yyyyMMdd') AS PUBLISH_DATE, to_char(a.EndDate, 'yyyyMMdd') AS END_DATE, OperatingRevenueGrowRate AS OPER_REVENUE_YOY, NetProfitGrowRate AS NET_PROFIT_YOY, ROETTM AS ROE_TTM, EPSTTM AS EPS_TTM, NetAssetPS AS NAPS, OperCashFlowPSTTM AS OCF_TTM, DividendTTM AS DIVIDEND_TTM FROM hsjy_gg.LC_MainIndexNew a LEFT JOIN hsjy_gg.SecuMain b ON a.CompanyCode=b.CompanyCode WHERE to_char(a.EndDate, 'yyyyMMdd')='{0}' and substr(b.SECUCODE, 0, 1) in ('0', '3', '6') and length(b.SECUCODE) = 6".format(date)
+        df = self.get_df(sql, db='readonly', page_size=200000)
+        return df
+
+    def read_star_stock_finance_jy(self, date):
+        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.InfoPublDate, 'yyyyMMdd') AS PUBLISH_DATE, to_char(a.EndDate, 'yyyyMMdd') AS END_DATE, OperatingRevenueYOY AS OPER_REVENUE_YOY, NetProfitYOY AS NET_PROFIT_YOY, ROETTM AS ROE_TTM, EPSTTM AS EPS_TTM, NetAssetPS AS NAPS, OperCashFlowPSTTM AS OCF_TTM, DividendTTM AS DIVIDEND_TTM from hsjy_gg.LC_STIBMainIndex a left join hsjy_gg.SecuMain b on a.CompanyCode=b.CompanyCode where to_char(a.EndDate, 'yyyyMMdd')='{0}' and substr(b.SECUCODE, 0, 1) in ('0', '3', '6') and length(b.SECUCODE) = 6 and a.IfMerged=1 and a.IfAdjusted=2".format(date)
+        df = self.get_df(sql, db='readonly', page_size=200000)
+        return df
+
+    def read_mutual_index_daily_k_given_indexs(self, indexs, start, end):
+        indexs = "'" + "','".join(indexs) + "'"
+        sql = "SELECT zsdm AS INDEX_SYMBOL, jzrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_fund.t_st_gm_clzs WHERE zsdm in ({0}) AND jzrq >= {1} AND jzrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
+        df = self.get_df(sql, db='funduser', page_size=200000)
+        return df
+
+    def read_private_index_daily_k_given_indexs(self, indexs, start, end):
+        indexs = "'" + "','".join(indexs) + "'"
+        sql = "SELECT zsdm AS INDEX_SYMBOL, jyrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_hedge.t_st_sm_zhmzs WHERE zsdm in ({0}) AND jyrq >= {1} AND jyrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
+        df = self.get_df(sql, db='highuser', page_size=200000)
+        return df
+
+    def read_fof_holding_given_codes(self, codes):
+        codes = "'" + "','".join(codes) + "'"
+        sql = "SELECT jjdm, jsrq, gbrq, cyjjdm, cyjjjc, cyjz, cyfe, zjbl FROM st_fund.t_st_gm_jjzh WHERE jjdm in ({0}) AND m_opt_type <> '03'".format(codes)
+        df = self.get_df(sql, db='funduser', page_size=200000)
+        return df
+
 if __name__ == '__main__':
     dates = HBDB().read_fund_barra_attribution_given_code('450004')
     barra_style_exposure = HBDB().read_barra_style_exposure_given_dates(fund_zc_wmv['RECENT_TRADE_DATE'].unique().tolist())
