@@ -394,7 +394,7 @@ class IndustryTracking:
         industry_head_pe_disp['PE_TTM_QUANTILE'] = industry_head_pe_disp['PE_TTM_QUANTILE'].apply(lambda x: '{0}%'.format(x))
         industry_head_pe_disp.columns = [industry, '股票代码', '股票名称', '市值（亿）', 'PE（TTM）', '历史分位水平', 'PE（TTM）FY0', 'PE（TTM）FY1', 'PE（TTM）FY2', 'PE（TTM）FY3']
         industry_head_pe_text = '以行业内市值排名前{0}的个股作为行业龙头个股，{1}内龙头个股的估值水平详见此excel：'.format(self.head, industry)
-        return industry_pe_quantile_disp, industry_pe_text, industry_cpe_text, industry_cpe_disp, industry_head_pe_text, industry_head_pe_disp
+        return industry_pe_latest, industry_pe_quantile_disp, industry_pe_text, industry_cpe_text, industry_cpe_disp, industry_head_pe_text, industry_head_pe_disp
 
     def get_industry_fmt(self, sw_type):
         industry = '申万一级行业' if sw_type == 1 else '申万二级行业' if sw_type == 2 else '申万三级关注行业'
@@ -505,7 +505,7 @@ class IndustryTracking:
         # document.add_picture('{0}sw{1}_wret.png'.format(self.file_path, 3))
 
         document.add_heading(text='二. 本周行业估值数据跟踪', level=1)
-        sw1_pe_quantile_disp, sw1_pe_text, sw1_cpe_text, sw1_cpe_disp, sw1_head_pe_text, sw1_head_pe_disp = self.get_industry_pe(1)
+        sw1_pe_latest, sw1_pe_quantile_disp, sw1_pe_text, sw1_cpe_text, sw1_cpe_disp, sw1_head_pe_text, sw1_head_pe_disp = self.get_industry_pe(1)
         document.add_paragraph(text=sw1_pe_text)
         document.add_picture('{0}sw{1}_pe.png'.format(self.file_path, 1))
         document.add_paragraph(text='注：蓝色点代表行业当前估值水平，黄色实线代表行业中位水平，绿色虚线代表行业平均水平。')
@@ -518,7 +518,7 @@ class IndustryTracking:
         document.add_paragraph(text=' ')
         p = document.add_paragraph(text=sw1_head_pe_text)
         add_hyperlink(p, '{0}行业数据_{1}.xlsx'.format(self.file_path, self.date), '行业数据。', '#3FAEEE', True)
-        # sw3_pe_quantile_disp, sw3_pe_text, sw3_cpe_text, sw3_cpe_disp, sw3_head_pe_text, sw3_head_pe_disp = self.get_industry_pe(3)
+        # sw3_pe_latest, sw3_pe_quantile_disp, sw3_pe_text, sw3_cpe_text, sw3_cpe_disp, sw3_head_pe_text, sw3_head_pe_disp = self.get_industry_pe(3)
         # document.add_paragraph(text=sw3_pe_text)
         # document.add_picture('{0}sw{1}_pe.png'.format(self.file_path, 3))
         # document.add_paragraph(text='注：蓝色点代表行业当前估值水平，黄色实线代表行业中位水平，绿色虚线代表行业平均水平。')
@@ -583,10 +583,43 @@ class IndustryTracking:
         plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
         plt.tight_layout()
         plt.savefig('{0}sw{1}_pe_roe.png'.format(self.file_path, 1))
-        sw1_pe_roe_text = '从下图可以看出，低估值水平高ROE行业主要有{0}。'.format('、'.join(sw1_pe_roe_select))
+        sw1_pe_roe_text = '从下图可以看出，估值低位且ROE水平相对较高的行业是我们重点关注的行业，包括{0}，这些行业的PE（TTM）历史分位水平小于50%，且ROE（TTM）在所有行业中处于前1/4的水平。'.format('、'.join(sw1_pe_roe_select))
         document.add_paragraph(text=sw1_pe_roe_text)
         document.add_picture('{0}sw{1}_pe_roe.png'.format(self.file_path, 1))
 
+        pe = sw1_pe_latest[['INDUSTRY_NAME', 'PE_TTM']].set_index('INDUSTRY_NAME')
+        pe['PE'] = pe['PE_TTM'].rank(ascending=False)
+        pe_q = sw1_pe_quantile_disp[['申万一级行业', '历史分位水平（%）']].set_index('申万一级行业')
+        pe_q['PE_Q'] = pe_q['历史分位水平（%）'].rank(ascending=False)
+        pe_score = pe[['PE']].merge(pe_q[['PE_Q']], left_index=True, right_index=True, how='left')
+        pe_score['PE_SCORE'] = (pe_score['PE'] + pe_score['PE_Q']) / 2.0
+        pe_score['PE_SCORE'] = pe_score['PE_SCORE'].rank()
+        fmt_score = sw1_fmt_disp.iloc[2:, [0, 7, 8, 9, 10, 11]]
+        fmt_score = fmt_score.set_index(0)
+        fmt_score.columns = ['NP_YOY', 'CNP_YOY', 'ROE', 'GIR', 'NPR']
+        for col in ['NP_YOY', 'CNP_YOY', 'ROE', 'GIR', 'NPR']:
+            fmt_score[col] = fmt_score[col].apply(lambda x: float(x.split('%')[0]))
+            fmt_score[col] = fmt_score[col].rank()
+        fmt_score['FMT_SCORE'] = fmt_score.mean(axis=1)
+        score = pe_score[['PE_SCORE']].merge(fmt_score[['FMT_SCORE']], left_index=True, right_index=True, how='left')
+        score['SCORE'] = (score['PE_SCORE'] + score['FMT_SCORE']) / 2.0
+        score['SCORE'] = score['SCORE'].rank()
+        score = score.sort_values('SCORE', ascending=False)
+        pe_induatry = list(score.sort_values('PE_SCORE', ascending=False).index)[:5]
+        fmt_induatry = list(score.sort_values('FMT_SCORE', ascending=False).index)[:5]
+        induatry = list(score.sort_values('SCORE', ascending=False).index)[:5]
+        score.columns = ['估值面打分', '基本面打分', '总分']
+        score = score.T.reset_index().T.reset_index()
+        score.iloc[0, 0] = ''
+        score_text = '从打分情况可以看出，估值维度打分前5的行业为{0}，基本面维度打分前5的行业为{1}，从总打分排名看，我们重点关注的行业为{2}'.format('、'.join(pe_induatry), '、'.join(fmt_induatry),'、'.join(induatry))
+        document.add_paragraph(text=score_text)
+        rowc, colc = score.shape[0], score.shape[1]
+        table = document.add_table(rows=rowc, cols=colc, style='Medium Grid 3')
+        for i in range(rowc):
+            for j in range(colc):
+                table.cell(i, j).text = str(score.iloc[i, j])
+        document.add_paragraph(text='')
+
         for paragraph in document.paragraphs:
             if paragraph.style.name == 'Normal' and paragraph.text != '':
                 paragraph.paragraph_format.first_line_indent = paragraph.style.font.size * 2
@@ -612,7 +645,9 @@ if __name__ == '__main__':
     run_date = week_trade_df[week_trade_df['TRADE_DATE'] <= today]['TRADE_DATE'].iloc[-1]
     last_run_date = week_trade_df[week_trade_df['TRADE_DATE'] <= today]['TRADE_DATE'].iloc[-2]
     if today == run_date:
-        file_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/'
+        file_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/{}'.format(run_date)
+        if not os.path.exists(file_path):
+            os.makedirs(file_path)
         head = 5
         report_date = '20220331'
         report_date_list = ['20191231', '20201231', '20210331', '20210630', '20210930', '20211231', '20220331']
