@@ -946,9 +946,12 @@ def fut_sec_stats_wind(
             ),
         ],
         legend_opts=opts.LegendOpts(
-            pos_top="7%",
-            pos_left='10%',
-            pos_right='10%'
+            orient="vertical",
+            type_='scroll',
+            pos_top="10%",
+            pos_left='77%',
+            pos_right='5%',
+            pos_bottom="5%",
         )
     )
     # web2.set_global_opts(
@@ -966,9 +969,10 @@ def fut_sec_stats_wind(
             ),
         ).add(
             web, grid_opts=opts.GridOpts(
-                pos_right="10%",
-                pos_left="10%",
-                pos_top='20%',
+                pos_right="25%",
+                pos_left="8%",
+                pos_top="10%",
+                pos_bottom="15%"
                 # pos_bottom='50%'
             ),
             # is_control_axis_index=True
