@@ -23,6 +23,7 @@ from hbshare.quant.Kevin.quant_room.IndexActiveStyleAnalyse import EquityIndexSt
 from hbshare.quant.Kevin.convertible_bond.CBMarketIV import CBMarketIV, CBMarketPrice, OptionIV
 from hbshare.quant.Kevin.asset_allocation.macro_index.stock_share_price_index_futures import StockSharePriceIndex
 from hbshare.quant.Kevin.asset_allocation.macro_index.stock_concept import StockConception
+from hbshare.quant.Kevin.quant_room.RiskControlReport import save_mkt_data_to_csv
 
 w.start()
 
@@ -96,9 +97,11 @@ def update_daily(start_date, end_date):
     StockSharePriceIndex(start_date, end_date).get_construct_result()
     # DR数据
     DRrates(start_date, end_date).get_construct_result()
+    # 市场数据存储
+    save_mkt_data_to_csv(start_date.replace('-', ''), end_date.replace('-', ''))
 
 
 if __name__ == '__main__':
-    update_daily(start_date='2022-10-13', end_date='2022-10-13')
-    # update_monthly('2022-07-10', '2022-09-30')
-    # update_weekly(start_date='2021-09-01', end_date='2022-09-30')
+    update_daily(start_date='2023-02-15', end_date='2023-02-15')
+    # update_monthly('2022-08-10', '2023-01-31')
+    # update_weekly(start_date='2021-09-01', end_date='2023-01-31')
