@@ -7,6 +7,8 @@ from datetime import datetime
 import hbshare as hbs
 from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql, get_trading_day_list
 from Arbitrage_backtest import cal_annual_return, cal_annual_volatility, cal_sharpe_ratio, cal_max_drawdown
+from hbshare.quant.CChen.fut import wind_stk_index_basis
+from hbshare.quant.CChen.cons import sql_write_path_hb
 import plotly
 from plotly.offline import plot as plot_ly
 import plotly.graph_objs as go
@@ -15,6 +17,53 @@ import plotly.figure_factory as ff
 plotly.offline.init_notebook_mode(connected=True)
 
 
+def alpha_extractor(start_date, end_date):
+    nav_series = get_fund_nav_from_sql(start_date, end_date, {"量锐62号": "SGR954"})
+    date_list = get_trading_day_list(start_date, end_date, frequency='week')
+    nav_series = nav_series.reindex(date_list)
+    # fut
+    start_dt = datetime.strptime(start_date, '%Y%m%d')
+    end_dt = datetime.strptime(end_date, '%Y%m%d')
+    sql_path = sql_write_path_hb['daily']
+    table = 'futures_wind'
+    data_00, data_01, data_02, data_03 = wind_stk_index_basis(
+        code='IC', start_date=start_dt, end_date=end_dt, sql_path=sql_path, table=table
+    )
+    data_01['t_date'] = data_01['t_date'].apply(lambda x: x.strftime('%Y%m%d'))
+    data_02['t_date'] = data_02['t_date'].apply(lambda x: x.strftime('%Y%m%d'))
+    fut_data = data_01[['t_date', 'basis']].rename(columns={"basis": "basis_01"}).set_index('t_date').merge(
+        data_02[['t_date', 'basis']].rename(columns={"basis": "basis_02"}).set_index('t_date'),
+        left_index=True, right_index=True)
+    fut_data = fut_data.reindex(date_list)
+
+    nav_series = nav_series.merge(fut_data, left_index=True, right_index=True)
+
+    sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
+                 "and JYRQ >= {} and JYRQ <= {}".format('000905', start_date, end_date)
+    res = hbs.db_data_query('readonly', sql_script, page_size=5000)
+    map_dict = {"000905": "中证500", "000300": "沪深300", "000852": "中证1000", "881001": "万得全A",
+                "000985": "中证全指"}
+    benchmark_name = map_dict['000905']
+    data = pd.DataFrame(res['data']).rename(columns={"TCLOSE": benchmark_name}).set_index(
+        'TRADEDATE')[[benchmark_name]]
+    benchmark_df = data.reindex(nav_series.index)
+
+    assert (nav_series.shape[0] == benchmark_df.shape[0])
+
+    nav_df = pd.merge(nav_series, benchmark_df, left_index=True, right_index=True)
+    nav_df['ret'] = nav_df['量锐62号'].pct_change()
+    nav_df['bm_ret'] = nav_df['中证500'].pct_change()
+    nav_df['excess'] = nav_df['ret'] - nav_df['bm_ret']
+    nav_df['delta_bs'] = nav_df['basis_02'].diff() / (100. * 52.)
+    nav_df = nav_df.dropna()
+    nav_df['actual_excess'] = (nav_df['excess'] - 0.2 * nav_df['delta_bs']) / 0.8
+
+    nav_df['alpha_actual'] = (1 + nav_df['excess']).cumprod()
+    nav_df['alpha_true'] = (1 + nav_df['actual_excess']).cumprod()
+
+    return nav_df
+
+
 class FundNavAnalysor:
     def __init__(self, start_date, end_date, benchmark_id, nav_series):
         self.start_date = start_date
@@ -174,8 +223,18 @@ if __name__ == '__main__':
     # nv_series = nv_series.set_index('trade_date')['天演300指增']
     # FundNavAnalysor('20191231', '20211231', '000300', nv_series).run()
 
-    nv_series = get_fund_nav_from_sql('20211224', '20220909', {"卓识全天候": "STJ403"})
-    period_date = get_trading_day_list('20211224', '20220909', frequency='week')
+    # alpha_extractor('20201231', '20221209')
+
+    s_date = "20190201"
+    e_date = "20230113"
+
+    nv_series = get_fund_nav_from_sql(s_date, e_date, {"赫富对冲三号": "SEU010"})
+    period_date = get_trading_day_list(s_date, e_date, frequency='week')
     nv_series = nv_series.reindex(period_date)
 
-    FundNavAnalysor('20211224', '20220909', '000985', nv_series).run()
\ No newline at end of file
+    # nv_series = pd.read_excel('D:\\玄信周度复权净值_20221118.xlsx', sheet_name=2, header=1)
+    # nv_series['净值日期'] = nv_series['净值日期'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
+    # nv_series.rename(columns={"复权单位净值": "玄信-招享1号"}, inplace=True)
+    # nv_series = nv_series.set_index('净值日期')[['玄信-招享1号']]
+
+    FundNavAnalysor(s_date, e_date, '000905', nv_series).run()
\ No newline at end of file
