@@ -168,7 +168,12 @@ alpha_series1 = Neutral2Long(nav_series1['概率一号'], "概率一号", 0.8).g
 # local_nav = get_fund_nav_from_sql('20210305', '20220415', {"星阔广厦": "SNU706"})
 
 from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql
-local_nav = get_fund_nav_from_sql('20210618', '20220729', {"概率": "SQQ803"})
+local_nav = get_fund_nav_from_sql('20210730', '20230113', {"伯兄": "SQT564"})
+
+nv_series = pd.read_excel('D:\\研究基地\\L-From_路遥\\1-跟踪代销\\星阔\\SSH984星阔广川21号中证500指数增强私募证券投资基金净值数据.xls', sheet_name=0, header=1)
+nv_series['估值日期'] = nv_series['估值日期'].apply(lambda x: x.replace('-', ''))
+nv_series.rename(columns={"累计单位净值": "星阔广川"}, inplace=True)
+local_nav = nv_series.set_index('估值日期')[['星阔广川']].sort_index().loc["20210910":]
 
 compare_dict = {"衍复": "SJH866",
                 "启林": "SGY379",
@@ -314,4 +319,4 @@ class LocalNavComparer:
 
 
 if __name__ == '__main__':
-    LocalNavComparer("20211231", "20220819").get_construct_result()
\ No newline at end of file
+    LocalNavComparer("20210910", "20230203").get_construct_result()
\ No newline at end of file
