@@ -546,4 +546,5 @@ if __name__ == '__main__':
     # ind_factor = IndustryCr(sdate, edate).calculate()
     # ind_factor.to_csv('D:\\市场微观结构图\\market_factor_save\\ind_cr.csv')
 
-    market_structure = MarketStructure()
\ No newline at end of file
+    # market_structure = MarketStructure()
+    MarketHist('20220429', '000905').daily_plot()
\ No newline at end of file
