@@ -79,6 +79,24 @@ def get_fund_nav_from_work(start_date, end_date, fund_id):
     return data
 
 
+def get_daily_nav_from_work(start_date, end_date, fund_dict):
+    nav_series_list = []
+    for name, fund_id in fund_dict.items():
+        sql_script = "SELECT * FROM daily_nav where code = '{}' and date >= '{}' and date <= '{}'".format(
+            fund_id, start_date, end_date)
+        engine = create_engine(engine_params)
+        data = pd.read_sql(sql_script, engine)
+        data['date'] = data['date'].map(str)
+
+        data = data.set_index('date')['fqjz']
+        data.name = name
+        nav_series_list.append(data)
+
+    df = pd.concat(nav_series_list, axis=1).sort_index()
+
+    return df
+
+
 def get_track_config_from_db():
     params_tmp = {
         "ip": "192.168.223.152",
