@@ -51,6 +51,13 @@ class HoldingAnalysor:
         holding_df = pd.read_sql(sql_script, engine)
         holding_df['trade_date'] = holding_df['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
 
+        # 招商量化精选 special
+        # sql_script = "SELECT jsrq as end_date, ggrq as report_date, zqdm as ticker, " \
+        #              "zjbl, zgbl, zlbl FROM st_fund.t_st_gm_gpzh where " \
+        #              "jjdm = '001917' and jsrq = {}".format(self.trade_date)
+        # holding_df = pd.DataFrame(hbs.db_data_query('funduser', sql_script, page_size=5000)['data'])
+        # holding_df.rename(columns={"end_date": "trade_date", "zjbl": "weight"}, inplace=True)
+
         # summary
         # sql_script = "SELECT * FROM private_fund_holding where trade_date = {}".format(self.trade_date)
         # engine = create_engine(engine_params)
@@ -433,13 +440,183 @@ class HoldingDistribution:
         return distribution_df
 
 
+class HoldingCompare:
+    def __init__(self, trade_date, benchmark_id, include_list):
+        self.trade_date = trade_date
+        self.benchmark_id = benchmark_id
+        self.include_list = include_list
+        self._load_data()
+
+    def _load_shift_date(self):
+        trade_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
+        pre_date = (trade_dt - datetime.timedelta(days=100)).strftime('%Y%m%d')
+
+        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
+            pre_date, self.trade_date)
+        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
+        df = pd.DataFrame(res['data']).rename(
+            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
+                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
+        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
+        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
+        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)
+
+        trading_day_list = df[df['isMonthEnd'] == 1]['calendarDate'].tolist()
+
+        return trading_day_list[-1]
+
+    def _load_portfolio_weight_series(self):
+        sql_script = "SELECT * FROM private_fund_holding where trade_date = {}".format(self.trade_date)
+        engine = create_engine(engine_params)
+        holding_df = pd.read_sql(sql_script, engine)
+        holding_df['trade_date'] = holding_df['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
+        holding_df = holding_df[holding_df['fund_name'].isin(self.include_list)]
+        # holding_df['total_weight'] = holding_df.groupby('fund_name')['weight'].transform('sum')
+        # holding_df['weight'] = holding_df['weight'] / holding_df['total_weight']
+
+        return holding_df[['fund_name', 'ticker', 'weight']]
+
+    def _load_benchmark_weight_series(self, date):
+        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(
+            self.benchmark_id)
+        res = hbs.db_data_query('readonly', sql_script)
+        index_info = pd.DataFrame(res['data'])
+        inner_code = index_info.set_index('SECUCODE').loc[self.benchmark_id, 'INNERCODE']
+
+        sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
+                     "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
+                     "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
+        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
+        weight_df = data.rename(
+            columns={"SECUCODE": "consTickerSymbol", "ENDDATE": "effDate", "WEIGHT": "weight"})
+
+        return weight_df.set_index('consTickerSymbol')['weight'] / 100.
+
+    @staticmethod
+    def _load_benchmark_components(date):
+        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and " \
+                     "SecuCode in ('000300', '000905', '000852')"
+        res = hbs.db_data_query('readonly', sql_script)
+        index_info = pd.DataFrame(res['data'])
+        inner_code_series = index_info.set_index('SECUCODE')['INNERCODE']
+
+        weight = []
+        for benchmark_id in ['000300', '000905', '000852']:
+            inner_code = inner_code_series.loc[benchmark_id]
+            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
+                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
+                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
+            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
+            weight_df = data.rename(
+                columns={"SECUCODE": "ticker", "ENDDATE": "effDate", "WEIGHT": "weight"})
+            weight_df['benchmark_id'] = benchmark_id
+            weight.append(weight_df[['ticker', 'benchmark_id']])
+
+        return pd.concat(weight)
+
+    @staticmethod
+    def _load_style_exposure(date):
+        sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(date)
+        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
+        exposure_df = pd.DataFrame(res['data']).set_index('ticker')
+        ind_names = [x.lower() for x in industry_name['sw'].values()]
+        exposure_df = exposure_df[style_name + ind_names]
+
+        return exposure_df
+
+    @staticmethod
+    def _load_stock_conception(date):
+        sql_script = "SELECT * FROM stock_conception where trade_date = {}".format(date)
+        engine = create_engine(engine_params)
+        concept_df = pd.read_sql(sql_script, engine)
+        concept_df['trade_date'] = concept_df['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
+        concept_df = concept_df[['trade_date', 'ticker', 'concept']]
+
+        return concept_df
+
+    def _load_data(self):
+        shift_date = self._load_shift_date()
+        portfolio_weight_series = self._load_portfolio_weight_series()
+        benchmark_weight_series = self._load_benchmark_weight_series(shift_date)
+        benchmark_components = self._load_benchmark_components(shift_date)
+        style_exposure_df = self._load_style_exposure(shift_date)
+        concept_df = self._load_stock_conception(shift_date)
+
+        self.data_param = {
+            "portfolio_weight_series": portfolio_weight_series,
+            "benchmark_weight_series": benchmark_weight_series,
+            "benchmark_components": benchmark_components,
+            "style_exposure_df": style_exposure_df,
+            "concept_df": concept_df
+        }
+
+    def run(self):
+        portfolio_weight_df = self.data_param.get('portfolio_weight_series')
+        portfolio_weight_df = pd.pivot_table(
+            portfolio_weight_df, index='ticker', columns='fund_name', values='weight').fillna(0.)
+        benchmark_weight_series = self.data_param.get('benchmark_weight_series')
+        benchmark_components = self.data_param.get('benchmark_components')
+        style_exposure_df = self.data_param.get('style_exposure_df')
+        concept_df = self.data_param.get('concept_df')
+
+        # 板块分布
+        weight_df = portfolio_weight_df.merge(
+            benchmark_weight_series.reset_index().rename(columns={"consTickerSymbol": "ticker", "weight": "bm"}),
+            on='ticker', how='outer').fillna(0.)
+        weight_df.loc[weight_df['ticker'].str.startswith('0'), 'sector'] = '深市'
+        weight_df.loc[weight_df['ticker'].str.startswith('60'), 'sector'] = '沪市'
+        weight_df.loc[weight_df['ticker'].str.startswith('30'), 'sector'] = '创业板'
+        weight_df.loc[weight_df['ticker'].str.startswith('688'), 'sector'] = '科创板'
+        sector_distribution = weight_df.groupby('sector')[self.include_list].sum()
+        # 成分股分布
+        w_df = pd.merge(portfolio_weight_df.reset_index(), benchmark_components, on='ticker', how='left')
+        w_df['benchmark_id'].fillna('other', inplace=True)
+        bm_distribution = w_df.groupby('benchmark_id').sum().reset_index().replace(
+            {"000300": "沪深300", "000905": "中证500", "000852": "中证1000", "other": "1800以外"}).set_index('benchmark_id')
+        # 风格及行业分布
+        idx = portfolio_weight_df.index.union(benchmark_weight_series.index).intersection(
+            style_exposure_df.index)
+
+        portfolio_weight_series = portfolio_weight_df.reindex(idx).fillna(0.)
+        benchmark_weight_series = benchmark_weight_series.reindex(idx).fillna(0.)
+        style_exposure_df = style_exposure_df.reindex(idx).astype(float)
+        portfolio_expo = style_exposure_df.T.dot(portfolio_weight_series)
+        benchmark_expo = style_exposure_df.T.dot(benchmark_weight_series)
+        style_expo = pd.concat([portfolio_expo, benchmark_expo.to_frame('bm')], axis=1)
+        style_expo = style_expo.sub(style_expo['bm'], axis=0)
+        # 概念
+        concept_df = concept_df.join(pd.get_dummies(concept_df['concept']))
+        concept_series = concept_df.groupby('ticker')[concept_df.columns[-8:]].sum()
+        idx = portfolio_weight_df.index.union(benchmark_weight_series.index).intersection(
+            concept_series.index)
+        portfolio_weight_series = portfolio_weight_df.reindex(idx).fillna(0.)
+        benchmark_weight_series = benchmark_weight_series.reindex(idx).fillna(0.)
+        concept_series = concept_series.reindex(idx).astype(float)
+        portfolio_concept = concept_series.T.dot(portfolio_weight_series)
+        benchmark_concept = concept_series.T.dot(benchmark_weight_series)
+        concept_expo = pd.concat([portfolio_concept, benchmark_concept.to_frame('bm')], axis=1)
+        concept_expo['active'] = concept_expo['port'] - concept_expo['bm']
+        # 持仓重合度
+        portfolio_weight_df = self.data_param.get('portfolio_weight_series')
+        portfolio_weight_df = pd.pivot_table(
+            portfolio_weight_df, index='ticker', columns='fund_name', values='weight')
+        overlap_df = pd.DataFrame(index=portfolio_weight_df.columns, columns=portfolio_weight_df.columns)
+        for i in range(len(portfolio_weight_df.columns)):
+            for j in range(len(portfolio_weight_df.columns)):
+                if i == j:
+                    continue
+                else:
+                    overlap_df.iloc[i, j] = portfolio_weight_df.T.iloc[[i, j], :].T.dropna().min(axis=1).sum()
+
+        return sector_distribution, bm_distribution, style_expo, concept_expo, overlap_df
+
+
 if __name__ == '__main__':
-    # d_list = ['20201231', '20210131', '20210228']
-    # tmp = HoldingAnalysor("星阔广厦1号中证500指数增强", "20220128", "000905").get_construct_result(isPlot=False)
+    tmp = HoldingAnalysor("顽岩中证500指数增强1号", "20221231", "000905").get_construct_result(isPlot=False)
 
-    # hd_analysor = HoldingAnalysor("量锐62号", "20220731", "000905")
-
-    tmp = HoldingAnalysor("稳博1000指数增强1号", "20220831", "000852").get_construct_result(isPlot=False)
-    print(tmp['style_df'])
     # dis_df = HoldingDistribution('20210310', '20220602', '因诺聚配中证500指数增强').get_construct_result()
-    # print(dis_df)
\ No newline at end of file
+    # print(dis_df)
+
+    # name_list = ['因诺聚配中证500指数增强', '星阔广厦1号中证500指数增强', '概率500指增2号', '乾象中证500指数增强1号',
+    #              '顽岩中证500指数增强1号', '伯兄熙宁', '赫富500指数增强一号']
+    # HoldingCompare('20221130', '000905', name_list).run()
\ No newline at end of file
