@@ -8,6 +8,7 @@ import os
 import plotly
 from plotly.offline import plot as plot_ly
 import plotly.graph_objs as go
+from Arbitrage_backtest import cal_annual_return
 
 plotly.offline.init_notebook_mode(connected=True)
 
@@ -34,8 +35,8 @@ class QuarterAssetAllocationReport:
         df = df[(df['month'] != df['next_month']) & (df['t_date'] >= self.start_date)]
         trading_day_list = sorted(df['t_date'].tolist())
 
-        # from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_trading_day_list
-        # trading_day_list = get_trading_day_list(self.start_date, self.end_date, frequency="week")
+        from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_trading_day_list
+        trading_day_list = get_trading_day_list(self.start_date, self.end_date, frequency="week")
 
         # type
         data_param = dict()
@@ -85,7 +86,7 @@ class QuarterAssetAllocationReport:
         return_300['ave_excess'] = return_300['average'] - return_300['benchmark']
 
         # 微调
-        data_1000.loc[:"20220211", "珠海宽德九睦"] = np.NaN
+        # data_1000.loc[:"20220211", "珠海宽德九睦"] = np.NaN
         return_1000 = data_1000.pct_change(limit=1).dropna(axis=0, how='all')
         return_1000['average'] = return_1000[return_1000.columns[:-1]].mean(axis=1)
         return_1000['ave_excess'] = return_1000['average'] - return_1000['benchmark']
@@ -106,6 +107,14 @@ class QuarterAssetAllocationReport:
             return_500['ave_excess'].to_frame('500指增'), left_index=True, right_index=True).merge(
             return_1000['ave_excess'].to_frame('1000指增'), left_index=True, right_index=True)
 
+        # if necessary
+        df = return_500[['average', 'benchmark', 'ave_excess']].copy()
+        df['超额净值'] = (1 + df['ave_excess']).cumprod()
+        df['超额最大回撤（右）'] = df['超额净值'] / df['超额净值'].cummax() - 1
+        df['滚动半年年化超额'] = df['ave_excess'].rolling(26).apply(cal_annual_return)
+        df['500指增平均绝对收益'] = (1 + df['average']).cumprod()
+        df['中证500'] = (1 + df['benchmark']).cumprod()
+
         return compare_df, compare_df2
 
     def calculate_index_return(self):
@@ -189,11 +198,8 @@ def plotly_line(df, day_list, title_text, save_path):
 
 if __name__ == '__main__':
     allo_rep = QuarterAssetAllocationReport(
-        start_date='20210930', end_date='20220930',
-        data_path='D:\\量化产品跟踪\\指数增强', file_name='指增-20220930.xlsx')
-    # allo_rep = QuarterAssetAllocationReport(
-    #     start_date='20161230', end_date='20220527',
-    #     data_path='D:\\量化产品跟踪\\指数增强', file_name='指增-20220527.xlsx')
+        start_date='20161230', end_date='20230203',
+        data_path='D:\\量化产品跟踪\\指数增强', file_name='指增-20230203.xlsx')
 
     excess_df, excess_df2 = allo_rep.calculate_excess_return()
 
