@@ -331,10 +331,11 @@ class AlphaBacktest:
 
         annual_alpha = group_ret_all.apply(cal_annual_return, axis=0)
         year_alpha = pd.DataFrame(
-            index=['2019', '2020', '2021'], columns=['group_{}'.format(i) for i in range(1, group_num + 1)] + ['port'])
+            index=['2019', '2020', '2021', '2022'], columns=['group_{}'.format(i) for i in range(1, group_num + 1)] + ['port'])
         year_alpha.loc['2019'] = (1 + group_ret_all[:"20191227"]).prod() - 1
         year_alpha.loc['2020'] = (1 + group_ret_all["20200103": "20201231"]).prod() - 1
-        year_alpha.loc['2021'] = (1 + group_ret_all["20210108":]).prod() - 1
+        year_alpha.loc['2021'] = (1 + group_ret_all["20210108": "20211231"]).prod() - 1
+        year_alpha.loc['2022'] = (1 + group_ret_all["20220107":]).prod() - 1
 
         weight_all = pd.concat(weight_list).reset_index()
         weight_all['weight'] = 1.
@@ -397,4 +398,4 @@ class AlphaBacktest:
 if __name__ == '__main__':
     # AlphaBacktest("D:\\量化产品跟踪\\指数增强", '20161230', '20220520').run()
     # f_df, g_ret = AlphaBacktest("D:\\量化产品跟踪\\指数增强", '20161230', '20220520').calculate_factor('20181228', 3, 3)
-    AlphaBacktest("D:\\量化产品跟踪\\指数增强", '20161230', '20220708').run('20181228')
\ No newline at end of file
+    AlphaBacktest("D:\\量化产品跟踪\\指数增强", '20161230', '20230113').run('20181228')
\ No newline at end of file
