@@ -42,6 +42,15 @@ class AlphaElasticity:
 
         nav_df = pd.concat(data_param, axis=1).dropna(how='all', axis=0).sort_index()
 
+        # 卓识的先手动入
+        nav_zs = pd.read_excel("D:\\卓识伟业净值\\卓识伟业_{}.xlsx".format(self.end_date), sheet_name=0)[
+            ['日期', '复权净值归一']].dropna()
+        nav_zs['日期'] = nav_zs['日期'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
+        nav_zs.rename(columns={"日期": "TRADEDATE", "复权净值归一": "卓识伟业"}, inplace=True)
+        nav_zs = nav_zs.set_index('TRADEDATE')['卓识伟业']
+        del nav_df['卓识伟业']
+        nav_df = nav_df.merge(nav_zs, left_index=True, right_index=True, how='left')
+
         sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                      "and JYRQ >= {} and JYRQ <= {}".format('000905', self.start_date, self.end_date)
         data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
@@ -112,4 +121,4 @@ class AlphaElasticity:
 
 
 if __name__ == '__main__':
-    AlphaElasticity("D:\\量化产品跟踪\\指数增强", '20220909').run()
\ No newline at end of file
+    AlphaElasticity("D:\\量化产品跟踪\\指数增强", '20221202').run()
\ No newline at end of file
