@@ -253,8 +253,8 @@ def daily_attr(start_date, end_date, mode="500指增"):
     # nav_df.loc[nav_df['诚奇睿盈500指数增强A'].isnull(), '诚奇睿盈500指数增强A'] = nav_df['诚奇睿盈500指数增强A-former']
 
     # 明汯1000的拼接
-    nav_df.loc[nav_df['明汯量化中小盘增强1号'].isnull(), '明汯量化中小盘增强1号'] = nav_df['明汯量化中小盘增强1号A']
-    nav_df.loc["20220826", "九坤日享中证1000指数增强1号"] = 3.8730
+    # nav_df.loc[nav_df['明汯量化中小盘增强1号'].isnull(), '明汯量化中小盘增强1号'] = nav_df['明汯量化中小盘增强1号A']
+    # nav_df.loc["20220826", "九坤日享中证1000指数增强1号"] = 3.8730
 
     nav_df = nav_df.dropna(axis=1).sort_index()
 
@@ -324,4 +324,4 @@ def daily_attr(start_date, end_date, mode="500指增"):
 
 if __name__ == '__main__':
     # AlphaNavAttribution('20201231', '20220819', '92c89fa3cc884540ba215920e005aa85')
-    daily_attr('20220801', '20220930', mode="1000指增")
+    daily_attr('20221202', '20230203', mode="其他")
