@@ -327,7 +327,13 @@ class MainFactorCrowd:
             ratio_list.append(ratio_df)
         t_ratio = pd.concat(ratio_list)
         # pre_ls_return
-        calc_df = p_factor_df.merge(week_ret, left_index=True, right_index=True)
+        sql_script = "SELECT * FROM factor_crowd where trade_date = '{}'".format(pre_week)
+        engine = create_engine(engine_params)
+        data = pd.read_sql(sql_script, engine)
+        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
+        pre_crowd_df = data.set_index('ticker')[['mean_to', 'stock_vol', 'beta']]
+        calc_df = p_factor_df.merge(week_ret, left_index=True, right_index=True).merge(
+            pre_crowd_df, left_index=True, right_index=True)
         ret_list = []
         for factor_name in style_list:
             ret_df = self.calc_ls_return(calc_df, factor_name)
@@ -432,10 +438,17 @@ if __name__ == '__main__':
     # for col in data1.columns:
     #     data1[col] = (data1[col] - data1[col].mean()) / data1[col].std()
 
-    sdate = '20101010'
-    # start_date = "20220902"
-    edate = '20220930'
-    date_list = get_trading_day_list(sdate, edate, frequency="week")
-    from tqdm import tqdm
-    for date in tqdm(date_list):
-        MainFactorCrowd(date).get_construct_result()
\ No newline at end of file
+    # sdate = '20101010'
+    # # start_date = "20220902"
+    # edate = '20220930'
+    # date_list = get_trading_day_list(sdate, edate, frequency="week")
+    # from tqdm import tqdm
+    # for date in tqdm(date_list):
+    #     MainFactorCrowd(date).get_construct_result()
+
+    "==================================================更新流程=========================================================="
+
+    date = "20230210"
+    FactorCrowdCalculator(date, r'D:\kevin\risk_model_jy\RiskModel\data\common_data').get_construct_result()
+    MainFactorCrowd(date).get_construct_result()
+
