@@ -8,6 +8,10 @@
 @file: fund_nav_test.py
 @time: 2020/6/15 10:50
 """
+import os, sys
+# 将hbshare根目录加入到环境变量
+BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+sys.path.append(BASE_DIR)
 
 import hbshare as hbs
 import time
@@ -30,6 +34,11 @@ def get_simu_nav_by_code():
     print(data)
     print(total_count)
 
+def get_prod_valuation_by_jjdm_gzrq():
+    hbs.set_token("qwertyuisdfghjkxcvbn1000")
+    data = hbs.get_prod_valuation_by_jjdm_gzrq("S29494", "20150723")
+    print(data)
+
 
 def test_get_fund_holding():
     hbs.set_token("qwertyuisdfghjkxcvbn1000")
@@ -49,9 +58,16 @@ def db_data_query():
     data = hbs.db_data_query('readonly', sql)
     print(data)
 
+def get_authorized_product_list():
+    hbs.set_token("qwertyuisdfghjkxcvbn1000")
+    mytoken ="..qRibJB_jTsGjtw3Anpua9n-"
+    data = hbs.get_authorized_product_list(mytoken,'false','1000')
+    print(data)
+
 
 if __name__ == "__main__":
-    db_data_query()
+    data = hbs.db_data_query('highuser',"select  jjdm,max(jzrq) as jzrq from st_hedge.t_st_sm_qjzdhc where zblb='2106' and jzrq>='20211201' and jzrq<='20211231' and zbnp!=99999 group by jjdm",timeout=300)
+    print(data)
     #test_get_fund_holding()
 
 
